@@ -16,6 +16,7 @@
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
+#include <sys/ioctl.h>
 
 /* for sendfile(2) */
 #ifdef __linux__
@@ -25,6 +26,7 @@
 #include <sys/uio.h>
 #endif
 
+#include "arg.h"
 #include "ind.h"
 #include "handlr.h"
 
@@ -52,14 +54,69 @@ filetype type[] = {
         {nil, nil, nil},
 };
 
+int
+pendingbytes(int sock)
+{
+	int pending, rval;
+
+	pending = 0;
+	rval = 0;
+#ifdef TIOCOUTQ
+	rval = ioctl(sock, TIOCOUTQ, &pending);
+#else
+#ifdef SIOCOUTQ
+	rval = ioctl(sock, SIOCOUTQ, &pending);
+#endif
+#endif
+
+	if (rval != 0)
+		return 0;
+
+	return pending;
+}
+
+void
+waitforpendingbytes(int sock)
+{
+	int npending, opending, trytime;
+
+	npending = opending = 0;
+	trytime = 10;
+
+	/*
+	 * Wait until there is nothing pending or the connection stalled
+	 * (nothing was sent) for around 40 seconds. Beware, trytime is
+	 * an exponential wait.
+	 */
+	while ((npending = pendingbytes(sock)) > 0 && trytime < 20000000) {
+		if (opending != 0) {
+			if (opending != npending) {
+				trytime = 10;
+			} else {
+				/*
+				 * Exponentially increase the usleep
+				 * waiting time to not waste CPU
+				 * resources.
+				 */
+				trytime += trytime;
+			}
+		}
+		opending = npending;
+
+		usleep(trytime);
+	}
+}
+
 int
 xsendfile(int fd, int sock)
 {
 	struct stat st;
-	char *sendb;
+	char *sendb, *sendi;
 	size_t bufsiz = BUFSIZ, count = 0;
 	int len, sent, optval;
 
+	USED(optval);
+
 /* Tell the kernel to not send small packets on every write. */
 #ifdef TCP_CORK
 	optval = 1;
@@ -97,13 +154,14 @@ xsendfile(int fd, int sock)
 	if (count == 0) {
 		sendb = xmalloc(bufsiz);
 		while ((len = read(fd, sendb, bufsiz)) > 0) {
+			sendi = sendb;
 			while (len > 0) {
-				if ((sent = send(sock, sendb, len, 0)) < 0) {
-					close(fd);
+				if ((sent = send(sock, sendi, len, 0)) < 0) {
 					free(sendb);
 					return -1;
 				}
 				len -= sent;
+				sendi += sent;
 			}
 		}
 		free(sendb);
@@ -239,7 +297,6 @@ getadv(char *str)
 	char *b, *e, *o, *bo;
 	Elems *ret;
 
-
 	ret = xcalloc(1, sizeof(Elems));
 
 	if (strchr(str, '\t')) {
@@ -364,20 +421,6 @@ printelem(int fd, Elems *el, char *addr, char *port)
 	return 0;
 }
 
-int
-initlogging(char *logf)
-{
-	return open(logf, O_APPEND | O_WRONLY | O_CREAT, 0644);
-}
-
-void
-stoplogging(int fd)
-{
-	close(fd);
-
-	return;
-}
-
 char *
 smprintf(char *fmt, ...)
 {
@@ -440,6 +483,8 @@ setcgienviron(char *file, char *path, char *port, char *base, char *args,
 	setenv("REMOTE_HOST", chost, 1);
 	unsetenv("REMOTE_IDENT");
 	unsetenv("REMOTE_USER");
+	/* Make PHP happy. */
+	setenv("REDIRECT_STATUS", "", 1);
 	/*
 	 * Only GET is possible in gopher. POST emulation would be really
 	 * ugly.
