@@ -39,13 +39,13 @@ char *xstrdup(const char *str);
 int xsendfile(int, int);
 Indexs *scanfile(char *fname);
 Elems *getadv(char *str);
+int pendingbytes(int sock);
+void waitforpendingbytes(int sock);
 int printelem(int fd, Elems *el, char *addr, char *port);
 void addindexs(Indexs *idx, Elems *el);
 void addelem(Elems *e, char *s);
 void freeindex(Indexs *i);
 void freeelem(Elems *e);
-int initlogging(char *logf);
-void stoplogging(int fd);
 char *smprintf(char *fmt, ...);
 char *reverselookup(char *host);
 void setcgienviron(char *file, char *path, char *port, char *base,
