@@ -39,7 +39,6 @@ enum {
 
 int glfd = -1;
 int loglvl = 15;
-int running = 1;
 int listfd = -1;
 int revlookup = 1;
 char *logfile = nil;
@@ -121,8 +120,11 @@ handlerequest(int sock, char *base, char *ohost, char *port, char *clienth,
 	args = nil;
 
 	len = recv(sock, recvb, sizeof(recvb)-1, 0);
-	if (len <= 0)
+	if (len <= 0) {
+		if (len < 0)
+			perror("recv");
 		return;
+	}
 
 	c = strchr(recvb, '\r');
 	if (c)
@@ -130,6 +132,27 @@ handlerequest(int sock, char *base, char *ohost, char *port, char *clienth,
 	c = strchr(recvb, '\n');
 	if (c)
 		c[0] = '\0';
+	sear = strchr(recvb, '\t');
+	if (sear != nil) {
+		*sear++ = '\0';
+
+		/*
+		 * This is a compatibility layer to geomyidae for users using
+		 * the original gopher(1) client. Gopher+ is by default
+		 * requesting the metadata. We are using a trick in the
+		 * gopher(1) parsing code to jump back to gopher compatibility
+		 * mode. DO NOT ADD ANY OTHER GOPHER+ SUPPORT. GOPHER+ IS
+		 * CRAP.
+		 */
+		if (*sear == '+' || *sear == '$' || *sear == '!' || *sear == '\0') {
+			dprintf(sock, "+-2\r\n");
+			dprintf(sock, "+INFO: 1gopher+\t\t%s\t%s\r\n",
+					ohost, port);
+			dprintf(sock, "+ADMIN:\r\n Admin: Me\r\n");
+			return;
+		}
+	}
+
 	memmove(recvc, recvb, len+1);
 
 	if (!strncmp(recvb, "URL:", 4)) {
@@ -143,9 +166,6 @@ handlerequest(int sock, char *base, char *ohost, char *port, char *clienth,
 		return;
 	}
 
-	sear = strchr(recvb, '\t');
-	if (sear != nil)
-		*sear++ = '\0';
 	args = strchr(recvb, '?');
 	if (args != nil)
 		*args++ = '\0';
@@ -225,8 +245,10 @@ sighandler(int sig)
 	case SIGABRT:
 	case SIGTERM:
 	case SIGKILL:
-		if (logfile != nil)
-			stoplogging(glfd);
+		if (logfile != nil && glfd != -1) {
+			close(glfd);
+			glfd = -1;
+		}
 		if (listfd >= 0) {
 			shutdown(listfd, SHUT_RDWR);
 			close(listfd);
@@ -275,6 +297,7 @@ getlistenfd(struct addrinfo *hints, char *bindip, char *port)
 			continue;
 		if (setsockopt(listfd, SOL_SOCKET, SO_REUSEADDR, &on,
 					sizeof(on)) < 0) {
+			close(listfd);
 			break;
 		}
 
@@ -384,6 +407,9 @@ main(int argc, char *argv[])
 		usage();
 	} ARGEND;
 
+	if (argc != 0)
+		usage();
+
 	if (ohost == nil) {
 		ohost = xcalloc(1, 513);
 		if (gethostname(ohost, 512) < 0) {
@@ -422,9 +448,9 @@ main(int argc, char *argv[])
 	}
 
 	if (logfile != nil) {
-		glfd = initlogging(logfile);
+		glfd = open(logfile, O_APPEND | O_WRONLY | O_CREAT, 0644);
 		if (glfd < 0) {
-			perror("initlogging");
+			perror("log");
 			return 1;
 		}
 	} else if (!dofork) {
@@ -489,17 +515,12 @@ main(int argc, char *argv[])
 	initsignals();
 
 	cltlen = sizeof(clt);
-	while (running) {
+	while (1) {
 		sock = accept(listfd, (struct sockaddr *)&clt, &cltlen);
 		if (sock < 0) {
 			switch (errno) {
 			case ECONNABORTED:
 			case EINTR:
-				if (!running) {
-					shutdown(listfd, SHUT_RDWR);
-					close(listfd);
-					return 0;
-				}
 				continue;
 			default:
 				perror("accept");
@@ -524,6 +545,8 @@ main(int argc, char *argv[])
 			shutdown(sock, SHUT_RDWR);
 			break;
 		case 0:
+			close(listfd);
+
 			signal(SIGHUP, SIG_DFL);
 			signal(SIGQUIT, SIG_DFL);
 			signal(SIGINT, SIG_DFL);
@@ -532,21 +555,28 @@ main(int argc, char *argv[])
 
 			handlerequest(sock, base, ohost, sport, clienth,
 						clientp);
+
+			waitforpendingbytes(sock);
+
 			shutdown(sock, SHUT_RDWR);
 			close(sock);
+
+			if (loglvl & CONN)
+				logentry(clienth, clientp, "-", "disconnected");
+
 			return 0;
 		default:
 			break;
 		}
 		close(sock);
-		if (loglvl & CONN)
-			logentry(clienth, clientp, "-", "disconnected");
 	}
 
 	shutdown(listfd, SHUT_RDWR);
 	close(listfd);
-	if (logfile != nil)
-		stoplogging(glfd);
+	if (logfile != nil && glfd != -1) {
+		close(glfd);
+		glfd = -1;
+	}
 	free(ohost);
 
 	return 0;
